import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:rook_sdk_core/rook_sdk_core.dart';
import 'package:rook_sdk_health_connect/rook_sdk_health_connect.dart';
import 'package:rook_sdk_health_connect/src/data/proto/protos.pb.dart';
import 'package:rook_sdk_health_connect/src/platform/rook_sdk_health_connect_method_channel.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  final platform = MethodChannelRookSdkHealthConnect();
  const channel = MethodChannel('rook_sdk_health_connect');

  tearDown(() {
    TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
        .setMockMethodCallHandler(channel, null);
  });

  group('MethodChannelRookSdkHealthConnect future completion', () {
    setUp(() {
      TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
          .setMockMethodCallHandler(channel, _happyHandler);
    });

    test('GIVEN the happy path WHEN setLocalLoggingLevel THEN complete',
        () async {
      final future = platform.setLocalLoggingLevel(LocalLoggingLevel.advanced);

      await expectLater(future, completes);
    });

    test('GIVEN the happy path WHEN setConfiguration THEN complete', () async {
      final rookConfiguration = RookConfiguration(
        'rookUrl',
        'clientUUID',
        'clientPassword',
      );
      final future = platform.setConfiguration(rookConfiguration);

      await expectLater(future, completes);
    });

    test('GIVEN the happy path WHEN initRook THEN complete', () async {
      final future = platform.initRook();

      await expectLater(future, completes);
    });

    test('GIVEN the happy path WHEN updateUserID THEN complete', () async {
      final future = platform.updateUserID('userID');

      await expectLater(future, completes);
    });

    test('GIVEN the happy path WHEN clearUserID THEN complete', () async {
      final future = platform.clearUserID();

      await expectLater(future, completes);
    });

    test(
      'GIVEN the happy path WHEN openHealthConnectSettings THEN complete',
      () async {
        final future = platform.openHealthConnectSettings();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN hasAllPermissions THEN complete with expected value',
      () async {
        final future = platform.hasAllPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN requestAllPermissions THEN complete',
      () async {
        final future = platform.requestAllPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN hasSleepPermissions THEN complete with expected value',
      () async {
        final future = platform.hasSleepPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN requestSleepPermissions THEN complete',
      () async {
        final future = platform.requestSleepPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN hasPhysicalPermissions THEN complete with expected value',
      () async {
        final future = platform.hasPhysicalPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN requestPhysicalPermissions THEN complete',
      () async {
        final future = platform.requestPhysicalPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN hasBodyPermissions THEN complete with expected value',
      () async {
        final future = platform.hasBodyPermissions();

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN requestBodyPermissions THEN complete',
      () async {
        final future = platform.requestBodyPermissions();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN shouldSyncSleepSummariesFor THEN complete with expected value',
      () async {
        final future = platform.shouldSyncSleepSummariesFor(DateTime.now());

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN syncSleepSummary THEN complete',
      () async {
        final future = platform.syncSleepSummary(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN shouldSyncPhysicalSummariesFor THEN complete with expected value',
      () async {
        final future = platform.shouldSyncPhysicalSummariesFor(DateTime.now());

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN syncSleepSummary THEN complete',
      () async {
        final future = platform.syncPhysicalSummary(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN shouldSyncBodySummariesFor THEN complete with expected value',
      () async {
        final future = platform.shouldSyncBodySummariesFor(DateTime.now());

        await expectLater(future, completion(true));
      },
    );

    test(
      'GIVEN the happy path WHEN syncBodySummary THEN complete',
      () async {
        final future = platform.syncBodySummary(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncPendingSummaries THEN complete',
      () async {
        final future = platform.syncPendingSummaries();

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncPhysicalEvents THEN complete',
      () async {
        final future = platform.syncPhysicalEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncBloodGlucoseEvents THEN complete',
      () async {
        final future = platform.syncBloodGlucoseEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncBloodPressureEvents THEN complete',
      () async {
        final future = platform.syncBloodPressureEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncBodyMetricsEvents THEN complete',
      () async {
        final future = platform.syncBodyMetricsEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncBodyHeartRateEvents THEN complete',
      () async {
        final future = platform.syncBodyHeartRateEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncPhysicalHeartRateEvents THEN complete',
      () async {
        final future = platform.syncPhysicalHeartRateEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncHydrationEvents THEN complete',
      () async {
        final future = platform.syncHydrationEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncNutritionEvents THEN complete',
      () async {
        final future = platform.syncNutritionEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncBodyOxygenationEvents THEN complete',
      () async {
        final future = platform.syncBodyOxygenationEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncPhysicalOxygenationEvents THEN complete',
      () async {
        final future = platform.syncPhysicalOxygenationEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncTemperatureEvents THEN complete',
      () async {
        final future = platform.syncTemperatureEvents(DateTime.now());

        await expectLater(future, completes);
      },
    );

    test(
      'GIVEN the happy path WHEN syncPendingEvents THEN complete',
      () async {
        final future = platform.syncPendingEvents();

        await expectLater(future, completes);
      },
    );
  });

  group('MethodChannelRookSdkHealthConnect future exception', () {
    setUp(() {
      TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
          .setMockMethodCallHandler(channel, _unhappyHandler);
    });

    test('GIVEN the unhappy path WHEN initRook THEN throw exception', () async {
      final future = platform.initRook();

      await expectLater(future, throwsA(isException));
    });

    test('GIVEN the unhappy path WHEN updateUserID THEN throw exception',
        () async {
      final future = platform.updateUserID('userID');

      await expectLater(future, throwsA(isException));
    });

    test(
      'GIVEN the unhappy path WHEN openHealthConnectSettings THEN throw exception',
      () async {
        final future = platform.openHealthConnectSettings();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN hasAllPermissions THEN throw exception',
      () async {
        final future = platform.hasAllPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN requestAllPermissions THEN throw exception',
      () async {
        final future = platform.requestAllPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN hasSleepPermissions THEN throw exception',
      () async {
        final future = platform.hasSleepPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN requestSleepPermissions THEN throw exception',
      () async {
        final future = platform.requestSleepPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN hasBodyPermissions THEN throw exception',
      () async {
        final future = platform.hasBodyPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN requestBodyPermissions THEN throw exception',
      () async {
        final future = platform.requestBodyPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN hasPhysicalPermissions THEN throw exception',
      () async {
        final future = platform.hasPhysicalPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN requestPhysicalPermissions THEN throw exception',
      () async {
        final future = platform.requestPhysicalPermissions();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN shouldSyncSleepSummariesFor THEN throw exception',
      () async {
        final future = platform.shouldSyncSleepSummariesFor(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncSleepSummary THEN complete',
      () async {
        final future = platform.syncSleepSummary(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN shouldSyncPhysicalSummariesFor THEN throw exception',
      () async {
        final future = platform.shouldSyncPhysicalSummariesFor(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncSleepSummary THEN complete',
      () async {
        final future = platform.syncPhysicalSummary(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN shouldSyncBodySummariesFor THEN throw exception',
      () async {
        final future = platform.shouldSyncBodySummariesFor(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncBodySummary THEN throw exception',
      () async {
        final future = platform.syncBodySummary(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncPendingSummaries THEN throw exception',
      () async {
        final future = platform.syncPendingSummaries();

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncPhysicalEvents THEN throw exception',
      () async {
        final future = platform.syncPhysicalEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncBloodGlucoseEvents THEN throw exception',
      () async {
        final future = platform.syncBloodGlucoseEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncBloodPressureEvents THEN throw exception',
      () async {
        final future = platform.syncBloodPressureEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncBodyMetricsEvents THEN throw exception',
      () async {
        final future = platform.syncBodyMetricsEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncBodyHeartRateEvents THEN throw exception',
      () async {
        final future = platform.syncBodyHeartRateEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncPhysicalHeartRateEvents THEN throw exception',
      () async {
        final future = platform.syncPhysicalHeartRateEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncHydrationEvents THEN throw exception',
      () async {
        final future = platform.syncHydrationEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncNutritionEvents THEN throw exception',
      () async {
        final future = platform.syncNutritionEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncBodyOxygenationEvents THEN throw exception',
      () async {
        final future = platform.syncBodyOxygenationEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncPhysicalOxygenationEvents THEN throw exception',
      () async {
        final future = platform.syncPhysicalOxygenationEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncTemperatureEvents THEN throw exception',
      () async {
        final future = platform.syncTemperatureEvents(DateTime.now());

        await expectLater(future, throwsA(isException));
      },
    );

    test(
      'GIVEN the unhappy path WHEN syncPendingEvents THEN throw exception',
      () async {
        final future = platform.syncPendingEvents();

        await expectLater(future, throwsA(isException));
      },
    );
  });

  group('MethodChannelRookSdkHealthConnect checkAvailability', () {
    group('Valid AvailabilityStatusProto value', () {
      setUp(() {
        TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
            .setMockMethodCallHandler(
          channel,
          (message) async {
            return AvailabilityStatusProto.INSTALLED.value;
          },
        );
      });

      test(
          'GIVEN the happy path WHEN checkAvailability THEN complete with expected value',
          () async {
        final future = platform.checkAvailability();

        await expectLater(future, completion(HCAvailabilityStatus.installed));
      });
    });

    group('Invalid AvailabilityStatusProto value', () {
      setUp(() {
        TestDefaultBinaryMessengerBinding.instance.defaultBinaryMessenger
            .setMockMethodCallHandler(
          channel,
          (message) async {
            return 1000;
          },
        );
      });

      test(
          'GIVEN the unhappy path WHEN checkAvailability THEN complete with HCAvailabilityStatus.notSupported',
          () async {
        final future = platform.checkAvailability();

        await expectLater(
            future, completion(HCAvailabilityStatus.notSupported));
      });
    });
  });
}

Future<Object?>? _happyHandler(MethodCall methodCall) async {
  final proto = ResultBooleanProto();
  proto.success = true;
  return proto.writeToBuffer();
}

Future<Object?>? _unhappyHandler(MethodCall methodCall) async {
  final proto = ResultBooleanProto();
  proto.genericExceptionProto = GenericExceptionProto(message: 'Generic error');

  return proto.writeToBuffer();
}
